@@ -100,6 +100,8 @@ struct auth_handler {
 
     ne_auth_creds creds;
     void *userdata;
+    int attempt; /* number of invocations of this callback for
+                  * current request. */
     
     struct auth_handler *next;
 };
@@ -324,7 +326,8 @@ static int get_credentials(auth_session *sess, int attempt,
                            struct auth_challenge *chall, char *pwbuf) 
 {
     return chall->handler->creds(chall->handler->userdata, sess->realm, 
-                                 attempt, sess->username, pwbuf);
+                                 chall->handler->attempt++, 
+                                 sess->username, pwbuf);
 }
 
 /* Examine a Basic auth challenge.
@@ -516,8 +519,9 @@ static int verify_negotiate_response(struct auth_request *req, auth_session *ses
     char *sep, *ptr = strchr(duphdr, ' ');
     int ret;
 
-    if (strncmp(hdr, "Negotiate", ptr - hdr) != 0) {
-        NE_DEBUG(NE_DBG_HTTPAUTH, "gssapi: Not a Negotiate response!\n");
+    if (strncmp(hdr, "Negotiate", ptr - duphdr) != 0) {
+        ne_set_error(sess->sess, _("Negotiate response verification failed: "
+                                   "invalid response header token"));
         ne_free(duphdr);
         return NE_ERROR;
     }
@@ -941,6 +945,7 @@ static int verify_digest_response(struct auth_request *req, auth_session *sess,
 	if (sess->nonce != NULL)
 	    ne_free(sess->nonce);
 	sess->nonce = ne_strdup(nextnonce);
+        sess->nonce_count = 0;
     }
 
     ne_free(hdr);
@@ -1131,6 +1136,7 @@ static void ah_create(ne_request *req, void *session, const char *method,
         (is_connect && sess->context == AUTH_CONNECT) ||
         (!is_connect && sess->context == AUTH_NOTCONNECT)) {
         struct auth_request *areq = ne_calloc(sizeof *areq);
+        struct auth_handler *hdl;
         
         NE_DEBUG(NE_DBG_HTTPAUTH, "ah_create, for %s\n", sess->spec->resp_hdr);
         
@@ -1139,6 +1145,12 @@ static void ah_create(ne_request *req, void *session, const char *method,
         areq->request = req;
         
         ne_set_request_private(req, sess->spec->id, areq);
+
+        /* For each new request, reset the attempt counter in every
+         * registered handler. */
+        for (hdl = sess->handlers; hdl; hdl = hdl->next) {
+            hdl->attempt = 0;
+        }
     }
 }
 
@@ -1200,7 +1212,7 @@ static int ah_post_send(ne_request *req, void *cookie, const ne_status *status)
         && (sess->protocol->flags & AUTH_FLAG_VERIFY_NON40x) == 0) {
         ret = sess->protocol->verify(areq, sess, auth_info_hdr);
     }
-    else if (sess->protocol
+    else if (sess->protocol && sess->protocol->verify
              && sess->protocol->flags && AUTH_FLAG_VERIFY_NON40x
              && (status->klass == 2 || status->klass == 3)
              && auth_hdr) {
@@ -1298,7 +1310,7 @@ static void auth_register(ne_session *sess, int isproxy, unsigned protomask,
 
     /* Find the end of the handler list, and add a new one. */
     hdl = &ahs->handlers;
-    while (*hdl && (*hdl)->next)
+    while (*hdl)
         hdl = &(*hdl)->next;
         
     *hdl = ne_malloc(sizeof **hdl);
@@ -1306,6 +1318,7 @@ static void auth_register(ne_session *sess, int isproxy, unsigned protomask,
     (*hdl)->creds = creds;
     (*hdl)->userdata = userdata;
     (*hdl)->next = NULL;
+    (*hdl)->attempt = 0;
 }
 
 static void auth_register_default(ne_session *sess, int isproxy,
