@@ -1,6 +1,6 @@
 /* 
    neon test suite
-   Copyright (C) 2002-2009, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 2002-2007, Joe Orton <joe@manyfish.co.uk>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -45,14 +45,10 @@
 #error SSL not supported
 #endif
 
-#include "ne_pkcs11.h"
 
 #define SERVER_CERT "server.cert"
-#define CA2_SERVER_CERT "ca2server.pem"
 #define CA_CERT "ca/cert.pem"
 
-#define P12_PASSPHRASE "foobar"
-
 #define SERVER_DNAME "Neon QA Dept, Neon Hackers Ltd, " \
                      "Cambridge, Cambridgeshire, GB"
 #define CACERT_DNAME "Random Dept, Neosign, Oakland, California, US"
@@ -63,8 +59,6 @@ static char *server_key = NULL;
 static ne_ssl_certificate *def_ca_cert = NULL, *def_server_cert;
 static ne_ssl_client_cert *def_cli_cert;
 
-static char *nul_cn_fn;
-
 static int check_dname(const ne_ssl_dname *dn, const char *expected,
                        const char *which);
 
@@ -77,7 +71,7 @@ struct ssl_server_args {
     /* client cert handling: */
     int require_cc; /* require a client cert if non-NULL */
     const char *ca_list; /* file of CA certs to verify client cert against */
-    int fail_silently; /* exit with success if handshake fails */
+    const char *send_ca; /* file of CA certs to send in client cert request */
     
     /* session caching: */
     int cache; /* use the session cache if non-zero */
@@ -88,8 +82,6 @@ struct ssl_server_args {
     int count; /* internal use. */
 
     int use_ssl2; /* force use of SSLv2 only */
-
-    const char *key;
 };
 
 /* default response string if args->response is NULL */
@@ -101,7 +93,6 @@ static int ssl_server(ne_socket *sock, void *userdata)
     struct ssl_server_args *args = userdata;
     int ret;
     char buf[BUFSIZ];
-    const char *key;
     static ne_ssl_context *ctx = NULL;
 
     if (ctx == NULL) {
@@ -111,25 +102,20 @@ static int ssl_server(ne_socket *sock, void *userdata)
 
     ONV(ctx == NULL, ("could not create SSL context"));
 
-    key = args->key ? args->key : server_key;
-    NE_DEBUG(NE_DBG_HTTP, "SSL server init with keypair (%s, %s).\n",
-             args->cert, key);
-             
-    ONV(ne_ssl_context_keypair(ctx, args->cert, key),
+    ONV(ne_ssl_context_keypair(ctx, args->cert, server_key),
         ("failed to load server keypair: ..."));
 
+    NE_DEBUG(NE_DBG_HTTP, "using server cert %s\n", args->cert);
+
     if (args->require_cc && !args->ca_list) {
         args->ca_list = CA_CERT;
     }
 
-    ne_ssl_context_set_verify(ctx, args->require_cc, 
-                              args->ca_list, args->ca_list);
+    ne_ssl_context_set_verify(ctx, args->require_cc, args->send_ca,
+                              args->ca_list);
 
-    ret = ne_sock_accept_ssl(sock, ctx);
-    if (ret && args->fail_silently) {
-        return 0;
-    }
-    ONV(ret, ("SSL accept failed: %s", ne_sock_error(sock)));
+    ONV(ne_sock_accept_ssl(sock, ctx),
+        ("SSL accept failed: %s", ne_sock_error(sock)));
 
     args->count++;
 
@@ -201,50 +187,6 @@ static int fail_serve(ne_socket *sock, void *ud)
 
 #define DEFSESS  (ne_session_create("https", "localhost", 7777))
 
-static int make_ssl_session_port(ne_session **sess,
-                                 const char *hostname, int port,
-                                 server_fn fn, void *userdata)
-{
-    return fakeproxied_session_server(sess, "https", hostname, port,
-                                      fn, userdata);
-}
-
-static int make_ssl_session(ne_session **sess, const char *hostname,
-                            server_fn fn, void *userdata)
-{
-    return make_ssl_session_port(sess,
-                                 hostname ? hostname : "localhost", 7777,
-                                 fn, userdata);
-}
-
-static int load_and_trust_cert(ne_session *sess, const char *ca_cert)
-{
-    ne_ssl_certificate *ca = ne_ssl_cert_read(ca_cert);
-    ONV(ca == NULL, ("could not load CA cert `%s'", ca_cert));
-    ne_ssl_trust_cert(sess, ca);
-    ne_ssl_cert_free(ca);
-    return OK;
-}
-
-static int make_ssl_request(struct ssl_server_args *args,
-                            const char *ca_cert, const char *hostname,
-                            ne_ssl_verify_fn verify_fn, void *verify_ud)
-{
-    ne_session *sess;
-    int ret;
-
-    CALL(make_ssl_session(&sess, hostname, ssl_server, args));
-
-    if (ca_cert) CALL(load_and_trust_cert(sess, ca_cert));
-    
-    if (verify_fn) ne_ssl_set_verify(sess, verify_fn, verify_ud);
-
-    ret = any_2xx_request(sess, "/foo");
-    CALL(await_server());
-    ne_session_destroy(sess);
-    return ret;
-}
-
 /* Run a request in the given session. */
 static int any_ssl_request(ne_session *sess, server_fn fn, void *server_ud,
 			   char *ca_cert,
@@ -253,7 +195,10 @@ static int any_ssl_request(ne_session *sess, server_fn fn, void *server_ud,
     int ret;
     
     if (ca_cert) {
-        CALL(load_and_trust_cert(sess, ca_cert));
+        ne_ssl_certificate *ca = ne_ssl_cert_read(ca_cert);
+        ONV(ca == NULL, ("could not load CA cert `%s'", ca_cert));
+        ne_ssl_trust_cert(sess, ca);
+        ne_ssl_cert_free(ca);
     }
 
     CALL(spawn_server(7777, fn, server_ud));
@@ -272,13 +217,13 @@ static int any_ssl_request(ne_session *sess, server_fn fn, void *server_ud,
 
 static int init(void)
 {
-    /* take srcdir as argv[1] for VPATH builds. */
-    if (test_argc > 1) {
-        srcdir = test_argv[1];
-    }
-        
     /* take srcdir as argv[1]. */
-    server_key = "server.key";
+    if (test_argc > 1) {
+	srcdir = test_argv[1];
+	server_key = ne_concat(srcdir, "/server.key", NULL);
+    } else {
+	server_key = "server.key";
+    }
     
     if (ne_sock_init()) {
 	t_context("could not initialize socket/SSL library.");
@@ -299,26 +244,13 @@ static int init(void)
     
     /* tests for the encrypted client cert, client.p12 */
     def_cli_cert = ne_ssl_clicert_read("client.p12");
-    if (def_cli_cert == NULL) {
-        t_context("could not load client.p12");
-        return FAILHARD;
-    }
+    ONN("could not load client.p12", def_cli_cert == NULL);
 
-    if (!ne_ssl_clicert_encrypted(def_cli_cert)) {
-        ne_ssl_clicert_free(def_cli_cert);
-        def_cli_cert = NULL;
-        t_context("client.p12 is not encrypted!?");
-        return FAIL;
-    }
-
-    if (ne_ssl_clicert_decrypt(def_cli_cert, P12_PASSPHRASE)) {
-        ne_ssl_clicert_free(def_cli_cert);
-        def_cli_cert = NULL;
-        t_context("failed to decrypt client.p12");
-        return FAIL;
-    }
-
-    nul_cn_fn = ne_concat(srcdir, "/nulcn.pem", NULL);
+    ONN("client.p12 is not encrypted!?", 
+        !ne_ssl_clicert_encrypted(def_cli_cert));
+    
+    ONN("failed to decrypt client.p12",
+        ne_ssl_clicert_decrypt(def_cli_cert, "foobar"));
 
     return OK;
 }
@@ -362,12 +294,9 @@ static int load_client_cert(void)
     ONN("could not load client.p12", cc == NULL);
     ONN("client.p12 not encrypted!?", !ne_ssl_clicert_encrypted(cc));
     name = ne_ssl_clicert_name(cc);
-    if (name == NULL) {
-        t_warning("no friendly name given");
-    } else {
-        ONV(strcmp(name, CC_NAME), ("friendly name was %s not %s", name, CC_NAME));
-    }
-    ONN("failed to decrypt", ne_ssl_clicert_decrypt(cc, P12_PASSPHRASE));
+    ONN("no friendly name given", name == NULL);
+    ONV(strcmp(name, CC_NAME), ("friendly name was %s not %s", name, CC_NAME));
+    ONN("failed to decrypt", ne_ssl_clicert_decrypt(cc, "foobar"));
     ne_ssl_clicert_free(cc);
 
     cc = ne_ssl_clicert_read("client.p12");
@@ -396,22 +325,13 @@ static int load_client_cert(void)
     ONV(name != NULL, ("noclient.p12 had friendly name `%s'", name));
     ne_ssl_clicert_free(cc);
 
-    /* test for ccert with a bundled CA. */
+    /* test for ccert without a friendly name, noclient.p12 */
     cc = ne_ssl_clicert_read("clientca.p12");
     ONN("could not load clientca.p12", cc == NULL);
     ONN("encrypted cert marked unencrypted?", !ne_ssl_clicert_encrypted(cc));
-    ONN("could not decrypt clientca.p12", 
-        ne_ssl_clicert_decrypt(cc, P12_PASSPHRASE));
+    ONN("could not decrypt clientca.p12", ne_ssl_clicert_decrypt(cc, "foobar"));
     ne_ssl_clicert_free(cc);
 
-    /* test for ccert without a private key, nkclient.p12 */
-    cc = ne_ssl_clicert_read("nkclient.p12");
-    ONN("did not fail to load clicert without pkey", cc != NULL);
-    
-    /* test for ccert without a cert, ncclient.p12 */
-    cc = ne_ssl_clicert_read("ncclient.p12");
-    ONN("did not fail to load clicert without cert", cc != NULL);
-
     /* tests for loading bogus files. */
     cc = ne_ssl_clicert_read("Makefile");
     ONN("loaded Makefile as client cert!?", cc != NULL);
@@ -423,31 +343,16 @@ static int load_client_cert(void)
     return OK;
 }
 
-static int clicert_import(void)
-{
-    ne_ssl_client_cert *cc;
-    ne_buffer *buf = ne_buffer_create();
-
-    CALL(file_to_buffer("client.p12", buf));
-
-    cc = ne_ssl_clicert_import((unsigned char *)buf->data, ne_buffer_size(buf));
-    ONN("could not import client cert from buffer", cc == NULL);
-    
-    ONN("failed to decrypt", ne_ssl_clicert_decrypt(cc, P12_PASSPHRASE));
-    ne_ssl_clicert_free(cc);
-    ne_buffer_destroy(buf);
-
-    return OK;
-}
-
 /* Test that 'cert', which is signed by CA_CERT, is accepted
- * unconditionally. */
+ * unconditionaly. */
 static int accept_signed_cert_for_hostname(char *cert, const char *hostname)
 {
+    ne_session *sess = ne_session_create("https", hostname, 7777);
     struct ssl_server_args args = {cert, 0};
-
     /* no verify callback needed. */
-    return make_ssl_request(&args, CA_CERT, hostname, NULL, NULL);
+    CALL(any_ssl_request(sess, ssl_server, &args, CA_CERT, NULL, NULL));
+    ne_session_destroy(sess);
+    return OK;
 }
 
 
@@ -461,56 +366,54 @@ static int simple(void)
     return accept_signed_cert(SERVER_CERT);
 }
 
-#if 0 /* No longer works for modern SSL libraries, rightly so. */
 /* Test for SSL operation when server uses SSLv2 */
 static int simple_sslv2(void)
 {
     ne_session *sess = ne_session_create("https", "localhost", 7777);
     struct ssl_server_args args = {SERVER_CERT, 0};
-
     args.use_ssl2 = 1;
     ne_set_session_flag(sess, NE_SESSFLAG_SSLv2, 1);
-
-    if (ne_get_session_flag(sess, NE_SESSFLAG_SSLv2) != 1) {
-        t_context("no SSLv2 support in SSL library");
-        ne_session_destroy(sess);
-        return SKIP;
-    }
-
     CALL(any_ssl_request(sess, ssl_server, &args, CA_CERT, NULL, NULL));
     ne_session_destroy(sess);
     return OK;
 }
-#endif
 
-/* Test read-til-EOF behaviour with SSL. */
-static int simple_eof(void)
+/* Serves using HTTP/1.0 get-till-EOF semantics. */
+static int serve_eof(ne_socket *sock, void *ud)
 {
-    struct ssl_server_args args = {SERVER_CERT, 0};
+    struct ssl_server_args args = {0};
 
+    args.cert = ud;
     args.response = "HTTP/1.0 200 OK\r\n"
         "Connection: close\r\n"
         "\r\n"
         "This is a response body, like it or not.";
 
-    return make_ssl_request(&args, CA_CERT, NULL, NULL, NULL);
+    return ssl_server(sock, &args);
 }
 
-static int intermediary(void)
+/* Test read-til-EOF behaviour with SSL. */
+static int simple_eof(void)
 {
-    struct ssl_server_args args = {CA2_SERVER_CERT, 0};
+    ne_session *sess = DEFSESS;
 
-    return make_ssl_request(&args, CA_CERT, NULL, NULL, NULL);
+    CALL(any_ssl_request(sess, serve_eof, SERVER_CERT, CA_CERT, NULL, NULL));
+    ne_session_destroy(sess);
+    return OK;
 }
 
 static int empty_truncated_eof(void)
 {
+    ne_session *sess = DEFSESS;
     struct ssl_server_args args = {0};
 
     args.cert = SERVER_CERT;
     args.response = "HTTP/1.0 200 OK\r\n" "\r\n";
     
-    return make_ssl_request(&args, CA_CERT, NULL, NULL, NULL);
+    CALL(any_ssl_request(sess, ssl_server, &args, CA_CERT, NULL, NULL));
+
+    ne_session_destroy(sess);
+    return OK;
 }
 
 /* Server function which just sends a string then EOF. */
@@ -524,10 +427,10 @@ static int just_serve_string(ne_socket *sock, void *userdata)
 /* test for the SSL negotiation failing. */
 static int fail_not_ssl(void)
 {
-    ne_session *sess;
+    ne_session *sess = DEFSESS;
     int ret;
     
-    CALL(make_ssl_session(&sess, NULL, just_serve_string, "Hello, world.\n"));
+    CALL(spawn_server(7777, just_serve_string, "Hello, world.\n"));
     ret = any_request(sess, "/bar");
     CALL(await_server());
 
@@ -537,18 +440,30 @@ static int fail_not_ssl(void)
     return OK;
 }
 
-static int wildcard_match(void)
+/* Server callback which handles a CONNECT request. */
+static int tunnel_server(ne_socket *sock, void *ud)
 {
-    struct ssl_server_args args = {"wildcard.cert", 0};
+    struct ssl_server_args *args = ud;
 
-    return make_ssl_request(&args, CA_CERT, "bar.example.com", NULL, NULL);
+    CALL(discard_request(sock));
+
+    SEND_STRING(sock, "HTTP/1.1 200 OK\r\nServer: serve_tunnel\r\n\r\n");
+    
+    return ssl_server(sock, args);
 }
 
-static int wildcard_match_altname(void)
+static int wildcard_match(void)
 {
-    struct ssl_server_args args = {"altname9.cert", 0};
+    ne_session *sess;
+    struct ssl_server_args args = {"wildcard.cert", 0};
+    
+    sess = ne_session_create("https", "anything.example.com", 443);
+    ne_session_proxy(sess, "localhost", 7777);
 
-    return make_ssl_request(&args, CA_CERT, "foo.example.com", NULL, NULL);
+    CALL(any_ssl_request(sess, tunnel_server, &args, CA_CERT, NULL, NULL));
+    ne_session_destroy(sess);
+    
+    return OK;
 }
 
 /* Check that hostname comparisons are not cases-sensitive. */
@@ -587,11 +502,6 @@ static int ipaddr_altname(void)
     return accept_signed_cert_for_hostname("altname5.cert", "127.0.0.1");
 }
 
-static int uri_altname(void)
-{
-    return accept_signed_cert_for_hostname("altname7.cert", "localhost");
-}
-
 /* test that the *most specific* commonName attribute is used. */
 static int multi_commonName(void)
 {
@@ -651,12 +561,15 @@ static int check_cert(void *userdata, int fs, const ne_ssl_certificate *cert)
 /* Check that certificate attributes are passed correctly. */
 static int parse_cert(void)
 {
-    struct ssl_server_args args = {SERVER_CERT, 0};
+    ne_session *sess = DEFSESS;
     int ret = 0;
+    struct ssl_server_args args = {SERVER_CERT, 0};
 
     /* don't give a CA cert; should force the verify callback to be
      * used. */
-    CALL(make_ssl_request(&args, NULL, NULL, check_cert, &ret));
+    CALL(any_ssl_request(sess, ssl_server, &args, NULL, 
+			 check_cert, &ret));
+    ne_session_destroy(sess);
 
     ONN("cert verification never called", ret == 0);
 
@@ -666,15 +579,12 @@ static int parse_cert(void)
     return OK;
 }
 
-#define WRONGCN_DNAME "Bad Hostname Department, Neon Hackers Ltd, " \
-    "Cambridge, Cambridgeshire, GB"
-
 /* Check the certificate chain presented against known dnames. */
 static int check_chain(void *userdata, int fs, const ne_ssl_certificate *cert)
 {
     int *ret = userdata;
 
-    if (check_cert_dnames(cert, WRONGCN_DNAME, CACERT_DNAME) == FAIL) {
+    if (check_cert_dnames(cert, SERVER_DNAME, CACERT_DNAME) == FAIL) {
         *ret = -1;
         return 0;
     }
@@ -698,14 +608,17 @@ static int check_chain(void *userdata, int fs, const ne_ssl_certificate *cert)
 /* Check that certificate attributes are passed correctly. */
 static int parse_chain(void)
 {
+    ne_session *sess = DEFSESS;
     int ret = 0;
-    struct ssl_server_args args = {"wrongcn.cert", 0};
+    struct ssl_server_args args = {SERVER_CERT, 0};
 
-    args.ca_list = CA_CERT;
+    args.ca_list = "ca/cert.pem";    
 
-    /* The cert is signed by the CA but has a CN mismatch, so will
-     * force the verification callback to be invoked. */
-    CALL(make_ssl_request(&args, CA_CERT, NULL, check_chain, &ret));
+    /* don't give a CA cert; should force the verify callback to be
+     * used. */
+    CALL(any_ssl_request(sess, ssl_server, &args, NULL, 
+			 check_chain, &ret));
+    ne_session_destroy(sess);
 
     ONN("cert verification never called", ret == 0);
 
@@ -725,28 +638,32 @@ static int count_vfy(void *userdata, int fs, const ne_ssl_certificate *c)
 
 static int no_verify(void)
 {
+    ne_session *sess = DEFSESS;
     int count = 0;
     struct ssl_server_args args = {SERVER_CERT, 0};
 
-    CALL(make_ssl_request(&args, CA_CERT, NULL, count_vfy, &count));
+    CALL(any_ssl_request(sess, ssl_server, &args, CA_CERT, count_vfy,
+			 &count));
 
     ONN("verify callback called unnecessarily", count != 0);
 
+    ne_session_destroy(sess);
+
     return OK;
 }
 
 static int cache_verify(void)
 {
     ne_session *sess = DEFSESS;
-    int count = 0;
+    int ret, count = 0;
     struct ssl_server_args args = {SERVER_CERT, 0};
     
     /* force verify cert. */
-    CALL(any_ssl_request(sess, ssl_server, &args, NULL, count_vfy,
-                         &count));
+    ret = any_ssl_request(sess, ssl_server, &args, NULL, count_vfy,
+			  &count);
 
     CALL(spawn_server(7777, ssl_server, &args));
-    ONREQ(any_request(sess, "/foo2"));
+    ret = any_request(sess, "/foo2");
     CALL(await_server());
 
     ONV(count != 1,
@@ -765,63 +682,21 @@ static int get_failures(void *userdata, int fs, const ne_ssl_certificate *c)
     return -1;
 }
 
-/* Helper function for expected-to-fail SSL tests.
- *
- * An SSL server is spawned using 'cert' and 'key' as the key pair.
- * The client will trust CA cert 'cacert', and use 'host' as the server
- * name.  If realhost is non-NULL, this address will be used to connect
- * to in favour of host; the server is otherwise identified as 'host'.
- * 'msg' must be a substring of the error string.
- * 'failures' must equal the failure bitmask passed to the verify
- * callback in the client.
- * If none of the expected conditions is met, 'errstr' will be
- * used in the test failure context.
- */
-static int fail_ssl_request_with_error2(char *cert, char *key, char *cacert, 
-                                        const char *host, const char *realhost,
-                                        const char *msg, int failures,
-                                        const char *errstr)
+/* Helper function: run a request using the given self-signed server
+ * certificate, and expect the request to fail with the given
+ * verification failure flags. */
+static int fail_ssl_request(char *cert, char *cacert, const char *host,
+			    const char *msg, int failures)
 {
     ne_session *sess = ne_session_create("https", host, 7777);
     int gotf = 0, ret;
-    struct ssl_server_args args = {0};
-    ne_sock_addr *addr = NULL;
-    const ne_inet_addr **list = NULL;
 
-    if (realhost) {
-        size_t n;
-        const ne_inet_addr *ia;
-
-        addr = ne_addr_resolve(realhost, 0);
-
-        ONV(ne_addr_result(addr),
-            ("fake hostname lookup failed for %s", realhost));
-
-        NE_DEBUG(NE_DBG_SSL, "ssl: Using fake hostname '%s'\n", realhost);
-
-        for (n = 0, ia = ne_addr_first(addr); ia; ia = ne_addr_next(addr))
-            n++;
-
-        NE_DEBUG(NE_DBG_SSL, "ssl: Address count '%lu'\n", n);
-
-        list = ne_calloc(n * sizeof(*list));
-
-        for (n = 0, ia = ne_addr_first(addr); ia; ia = ne_addr_next(addr))
-            list[n++] = ia;
-        
-        ne_set_addrlist(sess, list, n);
-    }
-
-    args.cert = cert;
-    args.key = key;
-    args.fail_silently = 1;
-    
-    ret = any_ssl_request(sess, ssl_server, &args, cacert,
+    ret = any_ssl_request(sess, fail_serve, cert, cacert,
 			  get_failures, &gotf);
 
     ONV(gotf == 0,
-	("no error in verification callback; request rv %d error string: %s",
-	 ret, ne_get_error(sess)));
+	("no error in verification callback; request failed: %s",
+	 ne_get_error(sess)));
 
     ONV(gotf & ~NE_SSL_FAILMASK,
 	("verification flags %x outside mask %x", gotf, NE_SSL_FAILMASK));
@@ -833,39 +708,11 @@ static int fail_ssl_request_with_error2(char *cert, char *key, char *cacert,
     /* and check that the request was failed too. */
     ONV(ret == NE_OK, ("%s", msg));
 
-    ONV(errstr && strstr(ne_get_error(sess), errstr) == NULL,
-        ("unexpected failure message '%s', wanted '%s'",
-         ne_get_error(sess), errstr));
-        
     ne_session_destroy(sess);
-    if (addr) ne_addr_destroy(addr);
-    if (list) ne_free(list);
 
     return OK;
 }
 
-/* Helper function: run a request using the given self-signed server
- * certificate, and expect the request to fail with the given
- * verification failure flags. */
-static int fail_ssl_request_with_error(char *cert, char *cacert, const char *host,
-                                       const char *msg, int failures,
-                                       const char *errstr)
-{
-    return fail_ssl_request_with_error2(cert, NULL, cacert, host, NULL,
-                                        msg, failures, errstr);
-}
-
-
-/* Helper function: run a request using the given self-signed server
- * certificate, and expect the request to fail with the given
- * verification failure flags. */
-static int fail_ssl_request(char *cert, char *cacert, const char *host,
-			    const char *msg, int failures)
-{
-    return fail_ssl_request_with_error(cert, cacert, host, msg, failures,
-                                       NULL);
-}        
-
 /* Note that the certs used for fail_* are mostly self-signed, so the
  * cert is passed as CA cert and server cert to fail_ssl_request. */
 
@@ -873,69 +720,37 @@ static int fail_ssl_request(char *cert, char *cacert, const char *host,
  * flagged as such. */
 static int fail_wrongCN(void)
 {
-    return fail_ssl_request_with_error("wrongcn.cert", "ca/cert.pem", "localhost",
-                                       "certificate with incorrect CN was accepted",
-                                       NE_SSL_IDMISMATCH,
-                                       "certificate issued for a different hostname");
-                            
-}
-
-#define SRCDIR(s) ne_concat(srcdir, "/" s, NULL)
-
-#ifndef HAVE_GNUTLS
-static int fail_nul_cn(void)
-{
-    char *key = SRCDIR("nulsrv.key"), *ca = SRCDIR("nulca.pem");
-    CALL(fail_ssl_request_with_error2(nul_cn_fn, key, ca,
-                                      "www.bank.com", "localhost",
-                                      "certificate with incorrect CN was accepted",
-                                      NE_SSL_IDMISMATCH|NE_SSL_EXPIRED|NE_SSL_BADCHAIN,
-                                      "certificate issued for a different hostname"));
-    ne_free(key);
-    ne_free(ca);
-    return OK;
-}
-#endif
-
-static int fail_nul_san(void)
-{
-    char *cert = SRCDIR("nulsan.pem"), *key = SRCDIR("nulsrv.key"),
-        *ca = SRCDIR("nulca.pem");
-    CALL(fail_ssl_request_with_error2(cert, key, ca, 
-                                      "www.bank.com", "localhost",
-                                      "certificate with incorrect CN was accepted",
-                                      NE_SSL_IDMISMATCH|NE_SSL_EXPIRED|NE_SSL_BADCHAIN,
-                                      "certificate issued for a different hostname"));
-    ne_free(cert);
-    ne_free(key);
-    ne_free(ca);
-    return OK;
+    return fail_ssl_request("wrongcn.pem", "wrongcn.pem", "localhost",
+			    "certificate with incorrect CN was accepted",
+			    NE_SSL_IDMISMATCH);
 }
 
 /* Check that an expired certificate is flagged as such. */
 static int fail_expired(void)
 {
-    return fail_ssl_request_with_error("expired.cert", CA_CERT,  "localhost",
-                                       "expired certificate was accepted", 
-                                       NE_SSL_EXPIRED,
-                                       "certificate has expired");
+    char *c = ne_concat(srcdir, "/expired.pem", NULL);
+    CALL(fail_ssl_request(c, c,  "localhost",
+                          "expired certificate was accepted", NE_SSL_EXPIRED));
+    ne_free(c);
+    return OK;
 }
 
 static int fail_notvalid(void)
 {
-    return fail_ssl_request_with_error("notyet.cert", CA_CERT,  "localhost",
-                                       "not yet valid certificate was accepted",
-                                       NE_SSL_NOTYETVALID,
-                                       "certificate is not yet valid");
+    char *c = ne_concat(srcdir, "/notvalid.pem", NULL);
+    CALL(fail_ssl_request(c, c,  "localhost",
+                          "not yet valid certificate was accepted",
+                          NE_SSL_NOTYETVALID));
+    ne_free(c);
+    return OK;    
 }
 
 /* Check that a server cert with a random issuer and self-signed cert
  * fail with UNTRUSTED. */
 static int fail_untrusted_ca(void)
 {
-    return fail_ssl_request_with_error("server.cert", NULL, "localhost",
-                                       "untrusted CA.", NE_SSL_UNTRUSTED,
-                                       "issuer is not trusted");
+    return fail_ssl_request("server.cert", NULL, "localhost",
+                            "untrusted CA.", NE_SSL_UNTRUSTED);
 }
 
 static int fail_self_signed(void)
@@ -976,60 +791,31 @@ static int fail_host_ipaltname(void)
                             "bad IP altname cert", NE_SSL_IDMISMATCH);
 }
 
-static int fail_bad_urialtname(void)
-{
-    return fail_ssl_request("altname8.cert", CA_CERT, "localhost",
-                            "bad URI altname cert", NE_SSL_IDMISMATCH);
-}
-
-static int fail_wildcard(void)
-{
-    return fail_ssl_request("altname9.cert", CA_CERT, "localhost",
-                            "subjaltname not honored", NE_SSL_IDMISMATCH);
-}
-
-static int fail_wildcard_ip(void)
-{
-    return fail_ssl_request("wildip.cert", CA_CERT, "127.0.0.1",
-                            "wildcard IP", NE_SSL_IDMISMATCH);
-}
-
-static int fail_ca_expired(void)
-{
-    return fail_ssl_request_with_error("ca1server.cert", "ca1/cert.pem", 
-                                       "localhost", "issuer ca expired", 
-                                       NE_SSL_BADCHAIN,
-                                       "bad certificate chain");
-}
-
-static int fail_ca_notyetvalid(void)
-{
-    return fail_ssl_request("ca3server.cert", "ca3/cert.pem", "localhost",
-                            "issuer ca not yet valid", NE_SSL_BADCHAIN);
-}
-
-#if 0
 /* Test that the SSL session is cached across connections. */
 static int session_cache(void)
 {
     struct ssl_server_args args = {0};
-    ne_session *sess;
-
+    ne_session *sess = ne_session_create("https", "localhost", 7777);
+    
     args.cert = SERVER_CERT;
     args.cache = 1;
 
-    CALL(multi_session_server(&sess, "https", "localhost",
-                              2, ssl_server, &args));
-
     ne_ssl_trust_cert(sess, def_ca_cert);
 
+    /* have spawned server listen for several connections. */
+    CALL(spawn_server_repeat(7777, ssl_server, &args, 4));
+
     ONREQ(any_request(sess, "/req1"));
     ONREQ(any_request(sess, "/req2"));
     ne_session_destroy(sess);
+    /* server should still be waiting for connections: if not,
+     * something went wrong. */
+    ONN("error from child", dead_server());
+    /* now get rid of it. */
+    reap_server();
 
-    return await_server();
+    return OK;
 }
-#endif
 
 /* Callback for client_cert_provider; takes a c. cert as userdata and
  * registers it. */
@@ -1052,7 +838,7 @@ static int client_cert_provided(void)
     cc = ne_ssl_clicert_read("client.p12");
     ONN("could not load client.p12", cc == NULL);
     ONN("could not decrypt client.p12", 
-        ne_ssl_clicert_decrypt(cc, P12_PASSPHRASE));
+        ne_ssl_clicert_decrypt(cc, "foobar"));
     
     ne_ssl_provide_clicert(sess, ccert_provider, cc);
     CALL(any_ssl_request(sess, ssl_server, &args, CA_CERT,
@@ -1063,14 +849,11 @@ static int client_cert_provided(void)
     return OK;
 }
 
-#define DN_COUNT 5
-
 static void cc_check_dnames(void *userdata, ne_session *sess,
                             const ne_ssl_dname *const *dns, int dncount)
 {
     int n, *ret = userdata;
-    static const char *expected[DN_COUNT] = {
-        CACERT_DNAME,
+    static const char *expected[4] = {
         "First Random CA, CAs Ltd., Lincoln, Lincolnshire, GB",
         "Second Random CA, CAs Ltd., Falmouth, Cornwall, GB",
         "Third Random CA, CAs Ltd., Ipswich, Suffolk, GB",
@@ -1079,14 +862,13 @@ static void cc_check_dnames(void *userdata, ne_session *sess,
 
     ne_ssl_set_clicert(sess, def_cli_cert);
 
-    if (dncount != DN_COUNT) {
-        t_context("dname count was %d not %d", dncount, 
-                  DN_COUNT);
+    if (dncount != 4) {
+        t_context("dname count was %d not 4", dncount);
         *ret = -1;
         return;
     }
     
-    for (n = 0; n < DN_COUNT; n++) {
+    for (n = 0; n < 4; n++) {
         char which[5];
 
         sprintf(which, "%d", n);
@@ -1108,7 +890,7 @@ static int cc_provided_dnames(void)
     struct ssl_server_args args = {SERVER_CERT, NULL};
 
     args.require_cc = 1;
-    args.ca_list = "calist.pem";
+    args.send_ca = "calist.pem";
 
     PRECOND(def_cli_cert);
 
@@ -1140,30 +922,6 @@ static int client_cert_pkcs12(void)
     return OK;
 }
 
-/* Test use of a PKCS#12 cert with an embedded CA cert - fails with <=
- * 0.28.3 in GnuTLS build. */
-static int client_cert_ca(void)
-{
-    ne_session *sess = DEFSESS;
-    struct ssl_server_args args = {SERVER_CERT, NULL};
-    ne_ssl_client_cert *cc;
-
-    args.require_cc = 1;
-
-    cc = ne_ssl_clicert_read("clientca.p12");
-    ONN("could not load clientca.p12", cc == NULL);
-    ONN("encrypted cert marked unencrypted?", !ne_ssl_clicert_encrypted(cc));
-    ONN("could not decrypt clientca.p12", 
-        ne_ssl_clicert_decrypt(cc, P12_PASSPHRASE));
-
-    ne_ssl_set_clicert(sess, cc);
-    CALL(any_ssl_request(sess, ssl_server, &args, CA_CERT, NULL, NULL));
-
-    ne_ssl_clicert_free(cc);
-
-    ne_session_destroy(sess);    
-    return OK;
-}
 
 /* Tests use of an unencrypted client certificate. */
 static int ccert_unencrypted(void)
@@ -1186,41 +944,6 @@ static int ccert_unencrypted(void)
     return OK;
 }
 
-#define NOCERT_MESSAGE "client certificate was requested"
-/* random SSL read may fail like this with TLSv1.3 */
-#define NOCERT_ALT "certificate required"
-
-/* Tests for useful error message if a handshake fails where a client
- * cert was requested. */
-static int no_client_cert(void)
-{
-    ne_session *sess = DEFSESS;
-    struct ssl_server_args args = {SERVER_CERT, NULL};
-    int ret;
-
-    args.require_cc = 1;
-    args.fail_silently = 1;
-
-    ne_ssl_trust_cert(sess, def_ca_cert);
-
-    CALL(spawn_server(7777, ssl_server, &args));
-    
-    ret = any_request(sess, "/failme");
-
-    ONV(ret != NE_ERROR,
-        ("unexpected result %d: %s", ret, ne_get_error(sess)));
-
-    ONV(strstr(ne_get_error(sess), NOCERT_MESSAGE) == NULL
-        && strstr(ne_get_error(sess), NOCERT_ALT) == NULL,
-        ("error message was '%s', missing '%s'", 
-         ne_get_error(sess), NOCERT_MESSAGE));
-    
-    reap_server();
-
-    ne_session_destroy(sess);    
-    return OK;
-}
-
 /* non-zero if a server auth header was received */
 static int got_server_auth; 
 
@@ -1389,7 +1112,7 @@ static int auth_tunnel_fail(void)
     ret = any_request(sess, "/bar");
     ONV(ret != NE_PROXYAUTH, ("bad error code for tunnel failure: %d", ret));
 
-    ONV(strstr(ne_get_error(sess), "GaBoogle") == NULL,
+    ONV(strstr(ne_get_error(sess), "Nyaaaaah") == NULL,
         ("bad error string for tunnel failure: %s", ne_get_error(sess)));
 
     ne_session_destroy(sess);
@@ -1457,7 +1180,6 @@ static int cert_identities(void)
         { "altname2.cert", "nohost.example.com" },
         { "altname4.cert", "localhost" },
         { "ca4.pem", "fourth.example.com" },
-        { "altname8.cert", "http://nohost.example.com/" },
         { NULL, NULL }
     };
     int n;
@@ -1468,22 +1190,6 @@ static int cert_identities(void)
     return OK;
 }
 
-static int nulcn_identity(void)
-{
-    ne_ssl_certificate *cert = ne_ssl_cert_read(nul_cn_fn);
-    const char *id;
-
-    ONN("could not read nulcn.pem", cert == NULL);
-
-    id = ne_ssl_cert_identity(cert);
-
-    ONN("embedded NUL byte not quoted",
-        id != NULL && strcmp(id, "www.bank.com") == 0);
-    
-    ne_ssl_cert_free(cert);
-    return OK;
-}
-
 static int check_validity(const char *fname,
                           const char *from, const char *until)
 {
@@ -1773,81 +1479,6 @@ static int cache_cert(void)
     return OK;
 }
 
-static int nonssl_trust(void)
-{
-    ne_session *sess = ne_session_create("http", "www.example.com", 80);
-    
-    ne_ssl_trust_cert(sess, def_ca_cert);
-    
-    ne_session_destroy(sess);
-
-    return OK;
-}
-
-/* PIN password provider callback. */
-static int pkcs11_pin(void *userdata, int attempt,
-                      const char *slot_descr, const char *token_label,
-                      unsigned int flags, char *pin)
-{
-    char *sekrit = userdata;
-
-    NE_DEBUG(NE_DBG_SSL, "pkcs11: slot = [%s], token = [%s]\n",
-             slot_descr, token_label);
-
-    if (attempt == 0) {
-        strcpy(pin, sekrit);
-        return 0;
-    }
-    else {
-        return -1;
-    }
-}
-
-static int nss_pkcs11_test(const char *dbname)
-{
-    ne_session *sess = DEFSESS;
-    struct ssl_server_args args = {SERVER_CERT, NULL};
-    ne_ssl_pkcs11_provider *prov;
-    int ret;
-
-    args.require_cc = 1;
-
-    if (access(dbname, R_OK|X_OK)) {
-        t_warning("NSS required for PKCS#11 testing");
-        return SKIP;
-    }
-
-    ret = ne_ssl_pkcs11_nss_provider_init(&prov, "softokn3", dbname, NULL, 
-                                          NULL, NULL);
-    if (ret) {
-        if (ret == NE_PK11_NOTIMPL)
-            t_context("pakchois library required for PKCS#11 support");
-        else
-            t_context("could not load NSS softokn3 PKCS#11 provider");
-        return SKIP;
-    }
-
-    ne_ssl_pkcs11_provider_pin(prov, pkcs11_pin, "foobar");
-    ne_ssl_set_pkcs11_provider(sess, prov);
-
-    ret = any_ssl_request(sess, ssl_server, &args, CA_CERT, NULL, NULL);
-
-    ne_session_destroy(sess);
-    ne_ssl_pkcs11_provider_destroy(prov);
-
-    return ret;
-}
-
-static int pkcs11(void)
-{
-    return nss_pkcs11_test("nssdb");
-}
-
-static int pkcs11_dsa(void)
-{
-    return nss_pkcs11_test("nssdb-dsa");
-}
-
 /* TODO: code paths still to test in cert verification:
  * - server cert changes between connections: Mozilla gives
  * a "bad MAC decode" error for this; can do better?
@@ -1855,7 +1486,6 @@ static int pkcs11_dsa(void)
  * only really happen if they mess with the SSL_CTX and enable
  * ADH cipher manually; but good to check the failure case is 
  * safe.
- * -  SSL cert changes between connections; handle as normal & re-verify
  * From the SSL book:
  * - an early FIN should be returned as a possible truncation attack,
  * NOT just an NE_SOCK_CLOSED.
@@ -1879,24 +1509,18 @@ ne_test tests[] = {
     T(read_write),
 
     T(load_client_cert),
-    T(clicert_import),
 
     T(simple),
-#if 0
     T(simple_sslv2),
-#endif
     T(simple_eof),
     T(empty_truncated_eof),
     T(fail_not_ssl),
     T(cache_cert),
-    T(intermediary),
 
     T(client_cert_pkcs12),
     T(ccert_unencrypted),
     T(client_cert_provided),
     T(cc_provided_dnames),
-    T(no_client_cert),
-    T(client_cert_ca),
 
     T(parse_cert),
     T(parse_chain),
@@ -1904,7 +1528,6 @@ ne_test tests[] = {
     T(no_verify),
     T(cache_verify),
     T(wildcard_match),
-    T(wildcard_match_altname),
     T(caseless_match),
 
     T(subject_altname),
@@ -1912,7 +1535,6 @@ ne_test tests[] = {
     T(two_subject_altname2),
     T(notdns_altname),
     T(ipaddr_altname),
-    T(uri_altname),
 
     T(multi_commonName),
     T(commonName_first),
@@ -1925,35 +1547,14 @@ ne_test tests[] = {
     T(fail_missing_CN),
     T(fail_host_ipaltname),
     T(fail_bad_ipaltname),
-    T(fail_bad_urialtname),
-    T(fail_wildcard),
-    T(fail_wildcard_ip),
-    T(fail_ca_notyetvalid),
-    T(fail_ca_expired),
 
-    T(nulcn_identity),
-#ifndef HAVE_GNUTLS
-    /* test failing with GnuTLS since GnuTLS fails handshake?
-     * fail_nul_cn (no error in verification callback; request rv 1 error string: Server certificate was missing commonName attribute in subject name)
-     */
-    T(fail_nul_cn),
-#endif
-    T(fail_nul_san),
-
-#if 0
     T(session_cache),
-#endif
-
+	
     T(fail_tunnel),
     T(proxy_tunnel),
     T(auth_proxy_tunnel),
     T(auth_tunnel_creds),
     T(auth_tunnel_fail),
 
-    T(nonssl_trust),
-
-    T(pkcs11),
-    T_XFAIL(pkcs11_dsa), /* unclear why this fails currently. */
-
     T(NULL) 
 };
